import { resolve } from 'path';
import { writeFile } from 'fs/promises';
import { PRPLCache, generateOrRetrieveFileSystemTree, log } from '@prpl/core';
import hljs from 'highlight.js/lib/core';
import { escape } from 'html-escaper';

var PRPLPluginCodeHighlightExtension;
(function (PRPLPluginCodeHighlightExtension) {
    PRPLPluginCodeHighlightExtension["html"] = ".html";
})(PRPLPluginCodeHighlightExtension || (PRPLPluginCodeHighlightExtension = {}));
var PRPLPluginCodeHighlightCachePartitionKey;
(function (PRPLPluginCodeHighlightCachePartitionKey) {
    PRPLPluginCodeHighlightCachePartitionKey["codeHighlight"] = "plugin-code-highlight";
})(PRPLPluginCodeHighlightCachePartitionKey || (PRPLPluginCodeHighlightCachePartitionKey = {}));
async function highlightCode(args) {
    const { cachePartitionKey } = args || {};
    if (!cachePartitionKey) {
        await PRPLCache?.define(PRPLPluginCodeHighlightCachePartitionKey.codeHighlight);
    }
    const partitionKey = cachePartitionKey || PRPLPluginCodeHighlightCachePartitionKey.codeHighlight;
    const distTree = await generateOrRetrieveFileSystemTree({
        entityPath: resolve('dist'),
        partitionKey,
        readFileRegExp: new RegExp(PRPLPluginCodeHighlightExtension.html)
    });
    async function walkDistTree(items) {
        for (let i = 0; i < items?.length; i++) {
            switch (items?.[i]?.entity) {
                case 'file':
                    try {
                        if (items?.[i]?.extension !== PRPLPluginCodeHighlightExtension.html) {
                            break;
                        }
                        const codeBlocks = [
                            ...items?.[i]?.src?.matchAll(/<code class="language-(.*?)">(.*?)<\/code>/gs)
                        ];
                        if (!codeBlocks?.length) {
                            break;
                        }
                        const mappedAliases = {
                            html: 'xml'
                        };
                        const escapedLanguages = {
                            xml: true
                        };
                        for (let [block, language, code] of codeBlocks) {
                            language = mappedAliases?.[language] ?? language;
                            const languagePath = `highlight.js/lib/languages/${language}`;
                            const languageRegistered = Boolean(hljs?.getLanguage(language));
                            if (!languageRegistered) {
                                const { default: languageGrammar } = await import(languagePath);
                                if (typeof languageGrammar !== 'function') {
                                    log.error(`Failed to load language grammar for ${language}`);
                                    return;
                                }
                                hljs?.registerLanguage(language, languageGrammar);
                            }
                            code = escapedLanguages?.[code] ? escape(code) : code;
                            const highlightedCode = hljs?.highlight(code, { language })?.value;
                            const reconstructedBlock = `<code class="language-${language}">${highlightedCode}</code>`;
                            items[i].src = items?.[i]?.src?.replace(block, reconstructedBlock);
                        }
                        await writeFile(items?.[i]?.targetFilePath, items?.[i]?.src);
                    }
                    catch (error) {
                        log.error(`Failed to highlight code block in file '${items?.[i]?.srcRelativeFilePath}'. Error:`, error?.message);
                    }
                    break;
                case 'directory':
                    await walkDistTree(items?.[i]?.children);
                    break;
            }
        }
    }
    await walkDistTree(distTree?.children || []);
    log.info('Highlighted code');
    return PRPLCache?.cache;
}

export { highlightCode };
